@@ -76,6 +76,9 @@
 #' passing the instance to `layer_torch_module_wrapper` (e.g. by calling
 #' it once).
 #'
+#' @param output_shape
+#' Shape of the output from this layer. Helps Keras infer shapes.
+#'
 #' @param name
 #' The name of the layer (string).
 #'
@@ -91,10 +94,11 @@
 #' @family layers
 #' @tether keras.layers.TorchModuleWrapper
 layer_torch_module_wrapper <-
-function (object, module, name = NULL, ...)
+function (object, module, output_shape = NULL, name = NULL, ...)
 {
     args <- capture_args(list(input_shape = normalize_shape,
-        batch_size = as_integer, batch_input_shape = normalize_shape),
+        batch_size = as_integer, batch_input_shape = normalize_shape,
+        output_shape = normalize_shape),
         ignore = "object")
     create_layer(keras$layers$TorchModuleWrapper, object, args)
 }
