@@ -647,6 +647,85 @@ function (y_true, y_pred, alpha = 0.25, gamma = 2,
 }
 
 
+
+#' Computes the generalized cross entropy loss.
+#'
+#' @description
+#' The generalized cross entropy (GCE) loss offers robustness to noisy labels by
+#' interpolating between categorical cross entropy (`q -> 0`) and mean absolute
+#' error (`q -> 1`). For a true-class probability `p` and noise parameter `q`,
+#' the loss is `loss = (1 - p^q) / q`.
+#'
+#' # References
+#' - Zhang & Sabuncu (2018), "Generalized Cross Entropy Loss for Training Deep
+#'   Neural Networks with Noisy Labels"
+#'
+#' # Examples
+#' ```{r}
+#' y_true <- c(0L, 1L, 0L, 1L)
+#' y_pred <- rbind(
+#'   c(0.7, 0.3),
+#'   c(0.2, 0.8),
+#'   c(0.6, 0.4),
+#'   c(0.4, 0.6)
+#' )
+#' gce <- loss_categorical_generalized_cross_entropy(q = 0.7)
+#' gce(y_true, y_pred)
+#' ```
+#'
+#' @returns
+#' Generalized cross entropy loss value(s).
+#'
+#' @param q
+#' Float in `(0, 1)`. Controls the transition between cross entropy and mean
+#' absolute error. Defaults to `0.5`.
+#'
+#' - As `q` approaches `0`: behaves like categorical cross entropy.
+#' - As `q` approaches `1`: behaves like mean absolute error.
+#'
+#' @param reduction
+#' Type of reduction to apply to the loss. In almost all cases
+#' this should be `"sum_over_batch_size"`. Supported options are
+#' `"sum"`, `"sum_over_batch_size"`, `"mean"`,
+#' `"mean_with_sample_weight"` or `NULL`. `"sum"` sums the loss,
+#' `"sum_over_batch_size"` and `"mean"` sum the loss and divide by the
+#' sample size, and `"mean_with_sample_weight"` sums the loss and
+#' divides by the sum of the sample weights. `"none"` and `NULL`
+#' perform no aggregation. Defaults to `"sum_over_batch_size"`.
+#'
+#' @param name
+#' Optional name for the loss instance.
+#'
+#' @param dtype
+#' Dtype used for loss computations. Defaults to `config_floatx()` (the global
+#' float type).
+#'
+#' @param y_true
+#' Integer class indices with shape `(batch_size)` or `(batch_size, 1)`.
+#'
+#' @param y_pred
+#' Predicted class probabilities with shape `(batch_size, num_classes)`.
+#'
+#' @param ...
+#' For forward/backward compatibility.
+#'
+#' @export
+#' @family losses
+#' @tether keras.losses.CategoricalGeneralizedCrossEntropy
+loss_categorical_generalized_cross_entropy <-
+function (y_true, y_pred, q = 0.5, ..., reduction = "sum_over_batch_size",
+          name = "categorical_generalized_cross_entropy", dtype = NULL)
+{
+    args <- capture_args(list(
+      y_true = as_py_array,
+      y_pred = as_py_array
+    ))
+    callable <- if (missing(y_true) && missing(y_pred))
+        keras$losses$CategoricalGeneralizedCrossEntropy
+    else keras$losses$categorical_generalized_cross_entropy
+    do.call(callable, args)
+}
+
 #' Computes the categorical hinge loss between `y_true` & `y_pred`.
 #'
 #' @description
@@ -820,12 +899,14 @@ function (y_true, y_pred, axis = -1L, ..., reduction = "sum_over_batch_size",
 #'                   1,   0, 1, 0.9), dim = c(2, 2, 2, 1))
 #'
 #' axis <- c(2, 3, 4)
-#' loss <- loss_dice(y_true, y_pred, axis = axis)
+#' loss_fn <- loss_dice(axis = axis, reduction = NULL)
+#' loss <- loss_fn(y_true, y_pred)
 #' stopifnot(shape(loss) == shape(2))
 #' loss
 #'
 #'
-#' loss = loss_dice(y_true, y_pred)
+#' loss_fn <- loss_dice()
+#' loss <- loss_fn(y_true, y_pred)
 #' stopifnot(shape(loss) == shape())
 #' loss
 #' ```
@@ -1539,37 +1620,34 @@ function (y_true, y_pred, ..., reduction = "sum_over_batch_size",
 #'
 #' # Examples
 #' ```{r}
-#' y_true <- c(1, 2)
-#' y_pred <- rbind(c(0.05, 0.95, 0), c(0.1, 0.8, 0.1))
+#' y_true <- op_array(c(1L, 2L))
+#' y_pred <- op_array(rbind(c(0.05, 0.95, 0), c(0.1, 0.8, 0.1)))
 #' loss <- loss_sparse_categorical_crossentropy(y_true, y_pred)
 #' loss
 #' ```
 #' ```{r}
-#' y_true <- c(1, 2)
-#' y_pred <- rbind(c(0.05, 0.95, 0), c(0.1, 0.8, 0.1))
+#' y_true <- op_array(c(1L, 2L))
+#' y_pred <- op_array(rbind(c(0.05, 0.95, 0), c(0.1, 0.8, 0.1)))
 #' # Using 'auto'/'sum_over_batch_size' reduction type.
 #' scce <- loss_sparse_categorical_crossentropy()
-#' scce(op_array(y_true), op_array(y_pred))
-#' # 1.177
+#' scce(y_true, y_pred)
 #' ```
 #'
 #' ```{r}
 #' # Calling with 'sample_weight'.
-#' scce(op_array(y_true), op_array(y_pred), sample_weight = op_array(c(0.3, 0.7)))
+#' scce(y_true, y_pred, sample_weight = op_array(c(0.3, 0.7)))
 #' ```
 #'
 #' ```{r}
 #' # Using 'sum' reduction type.
 #' scce <- loss_sparse_categorical_crossentropy(reduction="sum")
-#' scce(op_array(y_true), op_array(y_pred))
-#' # 2.354
+#' scce(y_true, y_pred)
 #' ```
 #'
 #' ```{r}
 #' # Using 'none' reduction type.
 #' scce <- loss_sparse_categorical_crossentropy(reduction=NULL)
-#' scce(op_array(y_true), op_array(y_pred))
-#' # array([0.0513, 2.303], dtype=float32)
+#' scce(y_true, y_pred)
 #' ```
 #'
 #' Usage with the `compile()` API:
