@@ -26,7 +26,7 @@ magrittr::`%<>%`
 #' @rdname multi-assign
 #' @keywords internal
 #' @returns The right-hand-side argument, `value`, invisibly. This called
-#'   primarily for it's side-effect of assigning symbols in the current frame.
+#'   primarily for its side-effect of assigning symbols in the current frame.
 #' @export
 #' @importFrom zeallot %<-%
 #' @usage x \%<-\% value
@@ -65,6 +65,21 @@ reticulate::iterate
 #' @export
 reticulate::as_iterator
 
+#' @export
+reticulate::py_require
+
+#' @export
+reticulate::import
+
+#' @export
+reticulate::py_to_r
+
+#' @export
+reticulate::r_to_py
+
+#' @export
+reticulate::py_help
+
 #' @importFrom tensorflow tensorboard
 #' @export
 tensorflow::tensorboard
@@ -113,6 +128,10 @@ generics::fit
 #' @export
 generics::compile
 
+#' @importFrom generics generate
+#' @export
+generics::generate
+
 # ' @importFrom generics evaluate
 # ' @export
 # generics::evaluate
